=== modified file 'bzrlib/tests/test_tuned_gzip.py'
--- bzrlib/tests/test_tuned_gzip.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_tuned_gzip.py	2009-06-02 19:56:24 +0000
@@ -85,3 +85,28 @@
         self.assertEqual('', stream.read())
         # and it should be new member time in the stream.
         self.failUnless(myfile._new_member)
+
+
+class TestToGzip(TestCase):
+
+    def assertToGzip(self, chunks):
+        bytes = ''.join(chunks)
+        gzfromchunks = tuned_gzip.chunks_to_gzip(chunks)
+        gzfrombytes = tuned_gzip.bytes_to_gzip(bytes)
+        self.assertEqual(gzfrombytes, gzfromchunks)
+        decoded = tuned_gzip.GzipFile(fileobj=StringIO(gzfromchunks)).read()
+        self.assertEqual(bytes, decoded)
+
+    def test_single_chunk(self):
+        self.assertToGzip(['a modest chunk\nwith some various\nbits\n'])
+
+    def test_simple_text(self):
+        self.assertToGzip(['some\n', 'strings\n', 'to\n', 'process\n'])
+
+    def test_large_chunks(self):
+        self.assertToGzip(['a large string\n'*1024])
+        self.assertToGzip(['a large string\n']*1024)
+
+    def test_enormous_chunks(self):
+        self.assertToGzip(['a large string\n'*1024*256])
+        self.assertToGzip(['a large string\n']*1024*256)

=== modified file 'bzrlib/tuned_gzip.py'
--- bzrlib/tuned_gzip.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tuned_gzip.py	2009-06-02 19:56:24 +0000
@@ -52,6 +52,18 @@
     width=-zlib.MAX_WBITS, mem=zlib.DEF_MEM_LEVEL,
     crc32=zlib.crc32):
     """Create a gzip file containing bytes and return its content."""
+    return chunks_to_gzip([bytes])
+
+
+def chunks_to_gzip(chunks, factory=zlib.compressobj,
+    level=zlib.Z_DEFAULT_COMPRESSION, method=zlib.DEFLATED,
+    width=-zlib.MAX_WBITS, mem=zlib.DEF_MEM_LEVEL,
+    crc32=zlib.crc32):
+    """Create a gzip file containing chunks and return its content.
+
+    :param chunks: An iterable of strings. Each string can have arbitrary
+        layout.
+    """
     result = [
         '\037\213'  # self.fileobj.write('\037\213')  # magic header
         '\010'      # self.fileobj.write('\010')      # compression method
@@ -69,11 +81,17 @@
     # using a compressobj avoids a small header and trailer that the compress()
     # utility function adds.
     compress = factory(level, method, width, mem, 0)
-    result.append(compress.compress(bytes))
+    crc = 0
+    total_len = 0
+    for chunk in chunks:
+        crc = crc32(chunk, crc)
+        total_len += len(chunk)
+        zbytes = compress.compress(chunk)
+        if zbytes:
+            result.append(zbytes)
     result.append(compress.flush())
-    result.append(struct.pack("<L", LOWU32(crc32(bytes))))
     # size may exceed 2GB, or even 4GB
-    result.append(struct.pack("<L", LOWU32(len(bytes))))
+    result.append(struct.pack("<LL", LOWU32(crc), LOWU32(total_len)))
     return ''.join(result)
 
 

